@@ -113,13 +113,16 @@ void qjackctlMainForm::init (void)
 
     // The eventual system tray widget.
     m_pSystemTray  = NULL;
-    
+
     // Working presets popup menu.
     m_pPresetsMenu = NULL;
-    
+
     // We're not quitting so early :)
     m_bQuitForce = false;
 
+	// Transport skip accelerate factor.
+	m_fSkipAccel = 1.0;
+
 #ifdef HAVE_SIGNAL_H
 	// Set to ignore any fatal "Broken pipe" signals.
 	signal(SIGPIPE, SIG_IGN);
@@ -136,10 +139,10 @@ void qjackctlMainForm::destroy (void)
     // Terminate local ALSA sequencer interface.
     if (m_pAlsaNotifier)
         delete m_pAlsaNotifier;
-
+#ifdef CONFIG_ALSA_SEQ
     if (m_pAlsaSeq)
         snd_seq_close(m_pAlsaSeq);
-
+#endif
     m_pAlsaNotifier = NULL;
     m_pAlsaSeq = NULL;
 
@@ -152,7 +155,7 @@ void qjackctlMainForm::destroy (void)
         delete m_pConnectionsForm;
     if (m_pPatchbayForm)
         delete m_pPatchbayForm;
-        
+
     // Quit off system tray widget.
     if (m_pSystemTray)
         delete m_pSystemTray;
@@ -221,7 +224,8 @@ bool qjackctlMainForm::setup ( qjackctlSetup *pSetup )
         m_pStdoutNotifier = new QSocketNotifier(g_fdStdout[QJACKCTL_FDREAD], QSocketNotifier::Read, this);
         QObject::connect(m_pStdoutNotifier, SIGNAL(activated(int)), this, SLOT(stdoutNotifySlot(int)));
     }
-
+    
+#ifdef CONFIG_ALSA_SEQ
     // Start our ALSA sequencer interface.
     if (snd_seq_open(&m_pAlsaSeq, "hw", SND_SEQ_OPEN_DUPLEX, 0) < 0)
         m_pAlsaSeq = NULL;
@@ -250,7 +254,6 @@ bool qjackctlMainForm::setup ( qjackctlSetup *pSetup )
             QObject::connect(m_pAlsaNotifier, SIGNAL(activated(int)), this, SLOT(alsaNotifySlot(int)));
         }
     }
-    
     // Could we start without it?
     if (m_pAlsaSeq == NULL) {
         appendMessagesError(tr("Could not open ALSA sequencer as a client.\n\nMIDI patchbay will be not available."));
@@ -261,14 +264,15 @@ bool qjackctlMainForm::setup ( qjackctlSetup *pSetup )
         if (m_pPatchbayForm)
             m_pPatchbayForm->setAlsaSeq(m_pAlsaSeq);
     }
-    
+#endif
+
     // Load patchbay from default path.
     if (m_pPatchbayForm && !m_pSetup->sPatchbayPath.isEmpty())
         m_pPatchbayForm->loadPatchbayFile(m_pSetup->sPatchbayPath);
 
     // Try to find if we can start in detached mode (client-only)
     // just in case there's a JACK server already running.
-    m_bJackDetach = startJackClient(true);
+    startJackClient(true);
     // Final startup stabilization...
     stabilizeForm();
     processJackExit();
@@ -279,7 +283,7 @@ bool qjackctlMainForm::setup ( qjackctlSetup *pSetup )
 
     // Register the first timer slot.
     QTimer::singleShot(QJACKCTL_TIMER_MSECS, this, SLOT(timerSlot()));
-    
+
     // We're ready to go...
     return true;
 }
@@ -403,7 +407,7 @@ QString qjackctlMainForm::formatExitStatus ( int iExitStatus )
         sTemp += tr("successfully");
     else
         sTemp += tr("with exit status=%1").arg(iExitStatus);
-    
+
     return sTemp + ".";
 }
 
@@ -470,10 +474,10 @@ void qjackctlMainForm::startJack (void)
     m_iTimerDelay  = 0;
     m_iJackRefresh = 0;
 
-    // If we ain't to be the server master...
-    if (m_bJackDetach) {
+    // If we ain't to be the server master,  maybe we'll start 
+	// detached as client only (jackd server already running?)
+    if (startJackClient(true)) {
         StopPushButton->setEnabled(true);
-        startJackClient(false);
         return;
     }
 
@@ -517,7 +521,7 @@ void qjackctlMainForm::startJack (void)
                 sCommand = fileinfo.filePath();
        }
     }
-    
+
     // Build process arguments...
     m_pJack->addArgument(sCommand);
     if (m_preset.bVerbose)
@@ -540,11 +544,14 @@ void qjackctlMainForm::startJack (void)
     bool bOss       = (m_preset.sDriver == "oss");
     bool bAlsa      = (m_preset.sDriver == "alsa");
     bool bPortaudio = (m_preset.sDriver == "portaudio");
+	bool bCoreaudio = (m_preset.sDriver == "coreaudio");
     if (bAlsa && (m_preset.iAudio != QJACKCTL_DUPLEX ||
 		m_preset.sInDevice.isEmpty() || m_preset.sOutDevice.isEmpty()))
         m_pJack->addArgument("-d" + m_preset.sInterface);
     if (bPortaudio && m_preset.iChan > 0)
         m_pJack->addArgument("-c" + QString::number(m_preset.iChan));
+	if (bCoreaudio)
+		m_pJack->addArgument("-n" + QString::number(m_preset.sInterface.toInt()));
     if (m_preset.iSampleRate > 0)
         m_pJack->addArgument("-r" + QString::number(m_preset.iSampleRate));
     if (m_preset.iFrames > 0)
@@ -570,10 +577,6 @@ void qjackctlMainForm::startJack (void)
             m_pJack->addArgument("-P" + m_preset.sOutDevice);
             break;
         }
-        if (m_preset.iInChannels > 0  && m_preset.iAudio != QJACKCTL_PLAYBACK)
-            m_pJack->addArgument("-i" + QString::number(m_preset.iInChannels));
-        if (m_preset.iOutChannels > 0 && m_preset.iAudio != QJACKCTL_CAPTURE)
-            m_pJack->addArgument("-o" + QString::number(m_preset.iOutChannels));
         if (m_preset.bSoftMode)
             m_pJack->addArgument("-s");
         if (m_preset.bMonitor)
@@ -581,6 +584,12 @@ void qjackctlMainForm::startJack (void)
         if (m_preset.bShorts)
             m_pJack->addArgument("-S");
     }
+	if (bAlsa || bCoreaudio) {
+		if (m_preset.iInChannels > 0  && m_preset.iAudio != QJACKCTL_PLAYBACK)
+			m_pJack->addArgument("-i" + QString::number(m_preset.iInChannels));
+		if (m_preset.iOutChannels > 0 && m_preset.iAudio != QJACKCTL_CAPTURE)
+			m_pJack->addArgument("-o" + QString::number(m_preset.iOutChannels));
+	}
     if (bOss) {
         if (m_preset.bIgnoreHW)
             m_pJack->addArgument("-b");
@@ -623,6 +632,12 @@ void qjackctlMainForm::startJack (void)
         if (m_preset.bHWMeter)
             m_pJack->addArgument("-M");
     }
+	if (bAlsa || bOss || bCoreaudio) {
+		if (m_preset.iInLatency > 0)
+			m_pJack->addArgument("-I" + QString::number(m_preset.iInLatency));
+		if (m_preset.iOutLatency > 0)
+			m_pJack->addArgument("-O" + QString::number(m_preset.iOutLatency));
+	}
 
     appendMessages(tr("JACK is starting..."));
     QStringList args = m_pJack->arguments();    
@@ -649,7 +664,7 @@ void qjackctlMainForm::startJack (void)
     // Sloppy boy fix: may the serve be stopped, just in case
     // the client will nerver make it...
     StopPushButton->setEnabled(true);
-    
+
     // Make sure all status(es) will be updated ASAP...
     m_iStatusRefresh += QJACKCTL_STATUS_CYCLE;
 
@@ -666,16 +681,16 @@ void qjackctlMainForm::stopJack (void)
 {
 	// Check if we're allowed to stop (shutdown)...
 	if (m_pSetup->bQueryShutdown
-	    && m_pJack && m_pJack->isRunning() && !m_bJackSurvive
-	    && m_pConnectionsForm && m_pConnectionsForm->isJackConnected()
-        && QMessageBox::warning(this, tr("Warning"),
-            tr("Some client audio applications") + "\n" +
+		&& m_pJack && m_pJack->isRunning() && !m_bJackSurvive
+		&& m_pConnectionsForm && m_pConnectionsForm->isJackConnected()
+		&& QMessageBox::warning(this, tr("Warning"),
+			tr("Some client audio applications") + "\n" +
 			tr("are still active and connected.") + "\n\n" +
-            tr("Do you want to stop the JACK audio server?"),
-            tr("Stop"), tr("Cancel")) > 0) {
+			tr("Do you want to stop the JACK audio server?"),
+			tr("Stop"), tr("Cancel")) > 0) {
 		return;
 	}
-
+	
 	// Stop the server unconditionally.
 	stopJackServer();
 }
@@ -726,7 +741,7 @@ void qjackctlMainForm::readJackStdout (void)
 void qjackctlMainForm::appendStdoutBuffer ( const QString& s )
 {
     m_sStdoutBuffer.append(s);
-    
+
     int iLength = m_sStdoutBuffer.findRev('\n') + 1;
     if (iLength > 0) {
         QString sTemp = m_sStdoutBuffer.left(iLength);
@@ -783,8 +798,11 @@ void qjackctlMainForm::processJackExit (void)
     ServerStateTextLabel->setPaletteForegroundColor(m_pJackClient == NULL ? Qt::darkYellow : Qt::yellow);
     StartPushButton->setEnabled(m_pJackClient == NULL);
     StopPushButton->setEnabled(m_pJackClient != NULL);
+    RewindPushButton->setEnabled(false);
+    BackwardPushButton->setEnabled(false);
     PlayPushButton->setEnabled(false);
     PausePushButton->setEnabled(false);
+    ForwardPushButton->setEnabled(false);
 }
 
 
@@ -950,10 +968,10 @@ void qjackctlMainForm::updateDisplayEffect (void)
     QPixmap pm;
     if (m_pSetup->bDisplayEffect)
         pm = QPixmap::fromMimeSource("displaybg1.png");
-        
+
     // Set the main origin...
     StatusDisplayFrame->setPaletteBackgroundPixmap(pm);
-    
+
     // Iterate for every child text label...
     QObjectList *pList = StatusDisplayFrame->queryList("QLabel");
     if (pList) {
@@ -1161,7 +1179,7 @@ void qjackctlMainForm::updateXrunCount (void)
         m_pSystemTray->setBackgroundMode(Qt::X11ParentRelative);
         m_pSystemTray->repaint(true);
     }
-        
+
     XrunCountTextLabel->setPaletteForegroundColor(color);
 
     QString sText = QString::number(m_iXrunCount);
@@ -1275,7 +1293,7 @@ void qjackctlMainForm::refreshXrunStats (void)
         updateStatusItem(STATUS_XRUN_AVG, QString::number(fXrunAverage) + s);
         updateStatusItem(STATUS_XRUN_LAST, QString::number(m_fXrunLast) + s);
     }
-    
+
     updateElapsedTimes();
 }
 
@@ -1393,11 +1411,11 @@ void qjackctlMainForm::shutNotifyEvent (void)
 // ALSA announce slot.
 void qjackctlMainForm::alsaNotifySlot ( int /*fd*/ )
 {
+#ifdef CONFIG_ALSA_SEQ
     snd_seq_event_t *pAlsaEvent;
-
     snd_seq_event_input(m_pAlsaSeq, &pAlsaEvent);
     snd_seq_free_event(pAlsaEvent);
-
+#endif
     // Log some message here, if new.
     if (m_iAlsaRefresh == 0)
         appendMessagesColor(tr("MIDI connection graph change."), "#66cc99");
@@ -1557,7 +1575,6 @@ bool qjackctlMainForm::startJackClient ( bool bDetach )
     }
 
     // Create the jack client handle, using a distinct identifier (PID?)
-    // surely
     QString sClientName = "qjackctl-" + QString::number((int) ::getpid());
     m_pJackClient = jack_client_new(sClientName.latin1());
     if (m_pJackClient == NULL) {
@@ -1600,9 +1617,11 @@ bool qjackctlMainForm::startJackClient ( bool bDetach )
         }
     }
 
-    // Do not forget to reset XRUN stats variables.
+   	// Do not forget to reset XRUN stats variables.
     if (!bDetach)
         resetXrunStats();
+	else // We'll flag that we've been detached!
+    	m_bJackDetach = true;
 
     // Activate us as a client...
     jack_activate(m_pJackClient);
@@ -1838,6 +1857,7 @@ void qjackctlMainForm::showSetupForm (void)
         bool    bOldStdoutCapture       = m_pSetup->bStdoutCapture;
         bool    bOldKeepOnTop           = m_pSetup->bKeepOnTop;
         bool    bOldSystemTray          = m_pSetup->bSystemTray;
+        bool    bOldDelayedSetup        = m_pSetup->bDelayedSetup;
         int     bOldMessagesLimit       = m_pSetup->bMessagesLimit;
         int     iOldMessagesLimitLines  = m_pSetup->iMessagesLimitLines;
         bool    bOldBezierLines         = m_pSetup->bBezierLines;
@@ -1886,7 +1906,9 @@ void qjackctlMainForm::showSetupForm (void)
             if (( bOldStdoutCapture && !m_pSetup->bStdoutCapture) ||
                 (!bOldStdoutCapture &&  m_pSetup->bStdoutCapture) ||
                 ( bOldKeepOnTop     && !m_pSetup->bKeepOnTop)     ||
-                (!bOldKeepOnTop     &&  m_pSetup->bKeepOnTop))
+                (!bOldKeepOnTop     &&  m_pSetup->bKeepOnTop)     ||
+                ( bOldDelayedSetup  && !m_pSetup->bDelayedSetup)  ||
+                (!bOldDelayedSetup  &&  m_pSetup->bDelayedSetup))
                 showDirtySetupWarning();
             // If server is currently running, warn user...
             showDirtySettingsWarning();
@@ -1907,8 +1929,45 @@ void qjackctlMainForm::showAboutForm (void)
 }
 
 
+// Transport rewind.
+void qjackctlMainForm::transportRewind (void)
+{
+#ifdef CONFIG_JACK_TRANSPORT
+	if (m_pJackClient) {
+		jack_transport_locate(m_pJackClient, 0);
+		// Log this here.
+		appendMessages(tr("Transport rewind."));
+		// Make sure all status(es) will be updated ASAP...
+		m_iStatusRefresh += QJACKCTL_STATUS_CYCLE;
+	}
+#endif
+}
+
+// Transport backward.
+void qjackctlMainForm::transportBackward (void)
+{
+#ifdef CONFIG_JACK_TRANSPORT
+	if (m_pJackClient) {
+		jack_position_t tpos;
+		jack_transport_query(m_pJackClient, &tpos);
+		float rate = (float) tpos.frame_rate;
+		float tloc = (((float) tpos.frame / rate) - m_fSkipAccel) * rate;
+		if (tloc < 0.0)	tloc = 0.0;
+		jack_transport_locate(m_pJackClient, (jack_nframes_t) tloc);
+		// Log this here (if on initial toggle).
+		if (m_fSkipAccel < 1.1) 
+			appendMessages(tr("Transport backward."));
+		// Take care of backward acceleration...
+		if (BackwardPushButton->isDown() && m_fSkipAccel < 60.0)
+			m_fSkipAccel *= 1.1;
+		// Make sure all status(es) will be updated ASAP...
+		m_iStatusRefresh += QJACKCTL_STATUS_CYCLE;
+	}
+#endif
+}
+
 // Transport start (play)
-void qjackctlMainForm::transportStart()
+void qjackctlMainForm::transportStart (void)
 {
 #ifdef CONFIG_JACK_TRANSPORT
     if (m_pJackClient) {
@@ -1923,7 +1982,7 @@ void qjackctlMainForm::transportStart()
 }
 
 // Transport stop (pause).
-void qjackctlMainForm::transportStop()
+void qjackctlMainForm::transportStop (void)
 {
 #ifdef CONFIG_JACK_TRANSPORT
     if (m_pJackClient) {
@@ -1937,6 +1996,29 @@ void qjackctlMainForm::transportStop()
 #endif
 }
 
+// Transport forward.
+void qjackctlMainForm::transportForward (void)
+{
+#ifdef CONFIG_JACK_TRANSPORT
+	if (m_pJackClient) {
+		jack_position_t tpos;
+		jack_transport_query(m_pJackClient, &tpos);
+		float rate = (float) tpos.frame_rate;
+		float tloc = (((float) tpos.frame / rate) + m_fSkipAccel) * rate;
+		if (tloc < 0.0)	tloc = 0.0;
+		jack_transport_locate(m_pJackClient, (jack_nframes_t) tloc);
+		// Log this here.
+		if (m_fSkipAccel < 1.1) 
+			appendMessages(tr("Transport forward."));
+		// Take care of forward acceleration...
+		if (ForwardPushButton->isDown() && m_fSkipAccel < 60.0)
+			m_fSkipAccel *= 1.1;
+		// Make sure all status(es) will be updated ASAP...
+		m_iStatusRefresh += QJACKCTL_STATUS_CYCLE;
+	}
+#endif
+}
+
 
 // Almost-complete running status refresher.
 void qjackctlMainForm::refreshStatus (void)
@@ -1946,7 +2028,7 @@ void qjackctlMainForm::refreshStatus (void)
     const QString sStopped = tr("Stopped");
 
     m_iStatusRefresh++;
-    
+
     if (m_pJackClient) {
         const QString s = " ";
 #ifdef CONFIG_JACK_TRANSPORT
@@ -1999,12 +2081,20 @@ void qjackctlMainForm::refreshStatus (void)
                 break;
             }
             updateStatusItem(STATUS_TRANSPORT_STATE, sText);
-            PlayPushButton->setEnabled(tstate == JackTransportStopped);
-            PausePushButton->setEnabled(bPlaying);
+			RewindPushButton->setEnabled(tpos.frame > 0);
+			BackwardPushButton->setEnabled(tpos.frame > 0);
+			PlayPushButton->setEnabled(tstate == JackTransportStopped);
+			PausePushButton->setEnabled(bPlaying);
+			ForwardPushButton->setEnabled(true);
+			if (!BackwardPushButton->isDown() && !ForwardPushButton->isDown())
+				m_fSkipAccel = 1.0;
 #else
             updateStatusItem(STATUS_TRANSPORT_STATE, n);
-            PlayPushButton->setEnabled(false);
-            PausePushButton->setEnabled(false);
+			RewindPushButton->setEnabled(false);
+			BackwardPushButton->setEnabled(false);
+			PlayPushButton->setEnabled(false);
+			PausePushButton->setEnabled(false);
+			ForwardPushButton->setEnabled(false);
             updateStatusItem(STATUS_TRANSPORT_TIME, m_sTimeDashes);
             updateStatusItem(STATUS_TRANSPORT_BBT, b);
             updateStatusItem(STATUS_TRANSPORT_BPM, n);
@@ -2031,8 +2121,11 @@ void qjackctlMainForm::refreshStatus (void)
         updateStatusItem(STATUS_TRANSPORT_TIME, m_sTimeDashes);
         updateStatusItem(STATUS_TRANSPORT_BBT, b);
         updateStatusItem(STATUS_TRANSPORT_BPM, n);
-        PlayPushButton->setEnabled(false);
-        PausePushButton->setEnabled(false);
+		RewindPushButton->setEnabled(false);
+		BackwardPushButton->setEnabled(false);
+		PlayPushButton->setEnabled(false);
+		PausePushButton->setEnabled(false);
+		ForwardPushButton->setEnabled(false);
     }
 
     // Elapsed times should be rigorous...
@@ -2086,7 +2179,7 @@ void qjackctlMainForm::updateStatusItem( int iStatusItem, const QString& sText )
 void qjackctlMainForm::updateTitleStatus (void)
 {
     QString sTitle = QJACKCTL_TITLE " ";
-    
+
     if (m_pSetup)
         sTitle += "[" + m_pSetup->sDefPreset + "] ";
 
@@ -2173,7 +2266,8 @@ void qjackctlMainForm::updateSystemTray (void)
 {
 #ifdef CONFIG_SYSTEM_TRAY
     if (!m_pSetup->bSystemTray && m_pSystemTray) {
-        m_pSystemTray->close();
+	//  Strange enough, this would close the application too.
+    //  m_pSystemTray->close();
         delete m_pSystemTray;
         m_pSystemTray = NULL;
     }
@@ -2253,7 +2347,7 @@ void qjackctlMainForm::systemTrayContextMenu ( const QPoint& pos )
     iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("patchbay1.png")),
         tr("Patch&bay"), this, SLOT(togglePatchbayForm()));
     pContextMenu->setItemChecked(iItemID, m_pPatchbayForm && m_pPatchbayForm->isVisible());
-    iItemID = pContextMenu->insertSeparator();
+    pContextMenu->insertSeparator();
 
     pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("setup1.png")),
         tr("S&etup..."), this, SLOT(showSetupForm()));
@@ -2265,7 +2359,7 @@ void qjackctlMainForm::systemTrayContextMenu ( const QPoint& pos )
         tr("&Quit"), this, SLOT(quitMainForm()));
 
     pContextMenu->exec(pos);
-    
+
     delete pContextMenu;
 
     delete m_pPresetsMenu;
