@@ -506,6 +506,8 @@ void qjackctlSocketList::clientPortsSnapshot (void)
         }
     }
 
+#ifdef CONFIG_ALSA_SEQ
+
     // Grab ALSA subscribers's...
     if (m_pAlsaSeq) {
         // Grab the ALSA subscriptions...
@@ -553,6 +555,8 @@ void qjackctlSocketList::clientPortsSnapshot (void)
             }
         }
     }
+
+#endif	// CONFIG_ALSA_SEQ
 }
 
 
@@ -689,6 +693,58 @@ bool qjackctlSocketList::editSocketItem (void)
 }
 
 
+// Duplicate and change the properties of currently selected socket item.
+bool qjackctlSocketList::copySocketItem (void)
+{
+    bool bResult = false;
+
+    qjackctlSocketItem *pSocketItem = selectedSocketItem();
+    if (pSocketItem) {
+        qjackctlSocketForm *pSocketForm = new qjackctlSocketForm(m_pListView);
+        if (pSocketForm) {
+			// Find a new distinguishable socket name, please.
+			QString sSocketName;
+			QString sSkel = pSocketItem->socketName();
+			sSkel.replace(QRegExp("[0-9]+$"), "%1");
+			int iSocketNo = 1;
+			do { sSocketName = sSkel.arg(++iSocketNo); }
+			while (findSocket(sSocketName));
+			// Show up as a new socket...
+            pSocketForm->setCaption(pSocketItem->socketName() + " <" + tr("Copy") + "> - " + m_sSocketCaption);
+            pSocketForm->setSocketCaption(m_sSocketCaption);
+            pSocketForm->setPixmaps(m_apPixmaps);
+            pSocketForm->setReadable(m_bReadable);
+            pSocketForm->setJackClient(m_pJackClient);
+            pSocketForm->setAlsaSeq(m_pAlsaSeq);
+	        qjackctlPatchbaySocket socket(sSocketName, pSocketItem->clientName(), pSocketItem->socketType());
+            if (pSocketItem->isExclusive())
+                socket.setExclusive(true);
+            for (qjackctlPlugItem *pPlugItem = pSocketItem->plugs().first(); pPlugItem; pPlugItem = pSocketItem->plugs().next())
+                socket.pluglist().append(pPlugItem->plugName());
+	        pSocketForm->load(&socket);
+	        if (pSocketForm->exec()) {	        
+	            pSocketForm->save(&socket);
+	            pSocketItem = new qjackctlSocketItem(this, socket.name(), socket.clientName(), socket.type(), socket.isExclusive(), pSocketItem);
+	            if (pSocketItem) {
+	                qjackctlPlugItem *pPlugItem = NULL;
+	                for (QStringList::Iterator iter = socket.pluglist().begin(); iter != socket.pluglist().end(); iter++)
+	                    pPlugItem = new qjackctlPlugItem(pSocketItem, *iter, pPlugItem);
+	                bResult = true;
+	            }
+	            m_pListView->setCurrentItem(pSocketItem);
+	            m_pListView->setSelected(pSocketItem, true);
+	        //  m_pListView->setUpdatesEnabled(true);
+	        //  m_pListView->triggerUpdate();
+	            m_pListView->setDirty(true);
+	        }
+	        delete pSocketForm;
+	    }
+	}    
+
+    return bResult;
+}           
+
+
 // Toggle exclusive currently selected socket item.
 bool qjackctlSocketList::exclusiveSocketItem (void)
 {
@@ -1153,32 +1209,44 @@ void qjackctlPatchbayView::contextMenu ( const QPoint& pos, qjackctlSocketList *
     if (pSocketList) {
         qjackctlSocketItem *pSocketItem = pSocketList->selectedSocketItem();
         bool bEnabled = (pSocketItem != NULL);
-        iItemID = pContextMenu->insertItem(tr("Add..."), pSocketList, SLOT(addSocketItem()));
-        iItemID = pContextMenu->insertItem(tr("Edit..."), pSocketList, SLOT(editSocketItem()));
+        iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("add1.png")),
+			tr("Add..."), pSocketList, SLOT(addSocketItem()));
+        iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("edit1.png")),
+			tr("Edit..."), pSocketList, SLOT(editSocketItem()));
         pContextMenu->setItemEnabled(iItemID, bEnabled);
-        iItemID = pContextMenu->insertItem(tr("Remove"), pSocketList, SLOT(removeSocketItem()));
+        iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("copy1.png")),
+			tr("Copy..."), pSocketList, SLOT(copySocketItem()));
+        pContextMenu->setItemEnabled(iItemID, bEnabled);
+        iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("remove1.png")),
+			tr("Remove"), pSocketList, SLOT(removeSocketItem()));
         pContextMenu->setItemEnabled(iItemID, bEnabled);
         pContextMenu->insertSeparator();
         iItemID = pContextMenu->insertItem(tr("Exclusive"), pSocketList, SLOT(exclusiveSocketItem()));
         pContextMenu->setItemChecked(iItemID, bEnabled && pSocketItem->isExclusive());
         pContextMenu->setItemEnabled(iItemID, bEnabled && (pSocketItem->connects().count() < 2));
         pContextMenu->insertSeparator();
-        iItemID = pContextMenu->insertItem(tr("Move Up"), pSocketList, SLOT(moveUpSocketItem()));
+        iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("up1.png")),
+			tr("Move Up"), pSocketList, SLOT(moveUpSocketItem()));
         pContextMenu->setItemEnabled(iItemID, (bEnabled && pSocketItem->itemAbove() != NULL));
-        iItemID = pContextMenu->insertItem(tr("Move Down"), pSocketList, SLOT(moveDownSocketItem()));
+        iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("down1.png")),
+			tr("Move Down"), pSocketList, SLOT(moveDownSocketItem()));
         pContextMenu->setItemEnabled(iItemID, (bEnabled && pSocketItem->nextSibling() != NULL));
         pContextMenu->insertSeparator();
     }
 
-    iItemID = pContextMenu->insertItem(tr("&Connect"), pPatchbay, SLOT(connectSelected()), tr("Alt+C", "Connect"));
+    iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("connect1.png")),
+		tr("&Connect"), pPatchbay, SLOT(connectSelected()), tr("Alt+C", "Connect"));
     pContextMenu->setItemEnabled(iItemID, pPatchbay->canConnectSelected());
-    iItemID = pContextMenu->insertItem(tr("&Disconnect"), pPatchbay, SLOT(disconnectSelected()), tr("Alt+D", "Disconnect"));
+    iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("disconnect1.png")),
+		tr("&Disconnect"), pPatchbay, SLOT(disconnectSelected()), tr("Alt+D", "Disconnect"));
     pContextMenu->setItemEnabled(iItemID, pPatchbay->canDisconnectSelected());
-    iItemID = pContextMenu->insertItem(tr("Disconnect &All"), pPatchbay, SLOT(disconnectAll()), tr("Alt+A", "Disconect All"));
+    iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("disconnectall1.png")),
+		tr("Disconnect &All"), pPatchbay, SLOT(disconnectAll()), tr("Alt+A", "Disconect All"));
     pContextMenu->setItemEnabled(iItemID, pPatchbay->canDisconnectAll());
 
     pContextMenu->insertSeparator();
-    iItemID = pContextMenu->insertItem(tr("&Refresh"), pPatchbay, SLOT(refresh()), tr("Alt+R", "Refresh"));
+    iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("refresh1.png")),
+		tr("&Refresh"), pPatchbay, SLOT(refresh()), tr("Alt+R", "Refresh"));
 
     pContextMenu->exec(pos);
     
@@ -1719,6 +1787,8 @@ void qjackctlPatchbay::socketPlugAudioSnapshot ( qjackctlSocketItem *pOSocket, q
 // MIDI connections snapshot.
 void qjackctlPatchbay::socketPlugMidiSnapshot ( qjackctlSocketItem *pOSocket, qjackctlPlugItem *pOPlug )
 {
+#ifdef CONFIG_ALSA_SEQ
+
     // ALSA sequencer descriptors...
     snd_seq_t *pAlsaSeq = alsaSeq();
 
@@ -1777,6 +1847,8 @@ void qjackctlPatchbay::socketPlugMidiSnapshot ( qjackctlSocketItem *pOSocket, qj
             }
         }
     }
+
+#endif	// CONFIG_ALSA_SEQ
 }
 
 
