@@ -66,6 +66,8 @@ struct qjackctlPreset
     QString sOutDevice;
     int     iInChannels;
     int     iOutChannels;
+    int     iInLatency;
+    int     iOutLatency;
     int     iStartDelay;
     bool    bVerbose;
     int     iPortMax;
@@ -130,6 +132,7 @@ public:
     bool    bQueryClose;
     bool    bKeepOnTop;
     bool    bSystemTray;
+    bool    bDelayedSetup;
     bool    bServerConfig;
     QString sServerConfigName;
     bool    bServerConfigTemp;
@@ -174,6 +177,30 @@ private:
 };
 
 
+// Delayed widget setup helper class.
+class qjackctlDelayedSetup : public QObject
+{
+    Q_OBJECT
+
+public:
+
+    // Constructor.
+    qjackctlDelayedSetup(QWidget *pWidget,
+        const QPoint& pos, const QSize& size, bool bVisible, int iDelay = 0);
+
+protected slots:
+
+     void setup();
+
+private:
+
+     QWidget *m_pWidget;
+     QPoint   m_pos;
+     QSize    m_size;
+     bool     m_bVisible;
+};
+
+
 #endif  // __qjackctlSetup_h
 
 
